@@ -1,6 +1,6 @@
 #!/usr/bin/perl
 
-# $Id: test_dbd_driver.pl,v 2.0 2002/03/19 04:00:04 lachoy Exp $
+# $Id: test_dbd_driver.pl,v 3.0 2002/08/28 01:16:31 lachoy Exp $
 
 # test_dbd_driver.pl
 
@@ -125,7 +125,7 @@ sub english_sql_type {
     return ( "SQL_BINARY", "blah" )       if ( $type == SQL_BINARY );
     return ( "SQL_VARBINARY", "blah" )    if ( $type == SQL_VARBINARY );
     return ( "SQL_LONGVARBINARY", "blah" ) if ( $type == SQL_LONGVARBINARY );
-    return ( "SQL_BIGINT", 4287 )         if ( $type == SQL_BIGINT );
+#    return ( "SQL_BIGINT", 4287 )         if ( $type == SQL_BIGINT );
     return ( "SQL_TINYINT", 42 )          if ( $type == SQL_TINYINT );
     return ( "SQL_BIT", 1 )               if ( $type == SQL_BIT );
     return ( "SQL_WCHAR", "wblah" )       if ( $type == SQL_WCHAR );
